<?xml version="1.0" encoding="UTF-8"?>
<gui name="rocs"
     version="10"
     xmlns="http://www.kde.org/standards/kxmlgui/1.0"
     xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
     xsi:schemaLocation="http://www.kde.org/standards/kxmlgui/1.0
                         http://www.kde.org/standards/kxmlgui/1.0/kxmlgui.xsd" >

  <MenuBar>
        <Menu name="file" >
            <Menu name="new">
                <text>New</text>
                <Action name="new-graph" />
                <Action name="new-script" />
            </Menu>
            <Action name="open-graph" />
            <Action name="open-script" />
            <Action name="save-all" />
            <Separator/>
            <Menu name="graph">
                <text>Graph</text>
                <Action name="save-graph" />
                <Action name="save-graph-as" />
                <Action name="import-graph" />
                <Action name="export-graph" />
            </Menu>
            <Menu name="script">
                <text>Script</text>
                <Action name="save-script" />
                <Action name="save-script-as" />
            </Menu>
            <Separator/>
            <Action name="download" />
            <Action name="upload" />
            <Action name="possible_includes" />
        </Menu>
        <Menu name="edit">
            <text>Edit</text>
            <Menu name="conversions">
                <text>Conversions</text>
                <ActionList name="DS_plugins" />
            </Menu>
            <Menu>
                <text>Layout</text>
                <Action name="align-htop"/>
                <Action name="align-hcenter"/>
                <Action name="align-hbottom"/>
                <Action name="align-vleft"/>
                <Action name="align-vcenter"/>
                <Action name="align-vright" />
                <Action name="align-circle" />
                <Action name="align-tree" />
            </Menu>
            <Action name="delete-selected" />
        </Menu>
        <Menu name="tools">
            <text>Tools</text>
            <ActionList name="tools_plugins" />
        </Menu>
    </MenuBar>

    <ToolBar name="main" >
        <text>Main Toolbar</text>
        <Action name="selectmove" />
        <Action name="add_node" />
        <Action name="add_edge" />
        <Action name="delete" />
        <Action name="zoom" />
    </ToolBar>
    <ToolBar name="layout">
        <text>Layout Toolbar</text>
        <Action name="align-htop"/>
        <Action name="align-hcenter"/>
        <Action name="align-hbottom"/>
        <Action name="align-vleft"/>
        <Action name="align-vcenter"/>
        <Action name="align-vright" />
        <Action name="align-circle" />
        <Action name="align-tree" />
    </ToolBar>
</gui>

