#include "rocs_KMLParserTest.h"
#include <QtTest/QTest>

void rocs_KMLParserTest::initTestCase()
{}

void rocs_KMLParserTest::init()
{}

void rocs_KMLParserTest::cleanup()
{}

void rocs_KMLParserTest::cleanupTestCase()
{}

void rocs_KMLParserTest::someTest()
{
    QWARN("A STUB!");
    QCOMPARE(1,1);
}


QTEST_MAIN(rocs_KMLParserTest)
#include "rocs_KMLParserTest.moc"
